// ==============================================================================================-
// EditChild.cpp : class for the ltl edit child windows
// ==============================================================================================-

#include "main.h"
#include "richedit.h"
#include "EditChild.h"


// ==============================================================================================-
// global data

EditChild *pEditChild[_MAX_OPEN_CHILDS];  //a pointer for every child window object

const char szFilters[] =
        "asm files\0*.asm\0" \
        "inc files\0*.inc\0" \
        "txt files\0*.txt\0" \
        "no filter\0*.*\0";

POINT pntNewWndPos = {0, 0};


// ==============================================================================================-
// global functions

// -----------------------------------------------------------------------------------------------
// initialize
void initEditChild()
{
  //set handles to zero
  memset(&pEditChild, 0, sizeof(pEditChild));
}

// -----------------------------------------------------------------------------------------------
// open dialog
BOOL showOpenDialog()
{
  OPENFILENAME ofn;

  ofn.lStructSize = sizeof(OPENFILENAME);
  ofn.hwndOwner = theApp.hwndMain;
  ofn.hInstance = theApp.hInstance;
  ofn.lpstrFilter = &szFilters[0];
  ofn.lpstrCustomFilter = NULL;
  ofn.nMaxCustFilter = 0;
  ofn.nFilterIndex = 0;

  ofn.nMaxFile = 1024;
  ofn.lpstrFile = (LPSTR)theApp.malloc(1024);
  if(!ofn.lpstrFile) return(-1);

  ofn.lpstrFileTitle = NULL; //(LPSTR)theApp.malloc(128);
  ofn.nMaxFileTitle = 0;
  ofn.lpstrInitialDir = NULL;
  ofn.lpstrTitle = NULL;
  ofn.Flags = OFN_ALLOWMULTISELECT|OFN_EXPLORER|OFN_LONGNAMES;  //|OFN_NOCHANGEDIR|OFN_SHAREAWARE
  ofn.nFileOffset = 0;
  ofn.nFileExtension = 0;
  ofn.lpstrDefExt = "asm";
  ofn.lCustData = NULL;
  ofn.lpfnHook = NULL;
  ofn.lpTemplateName = NULL;

  if(!GetOpenFileName( &ofn ) || !*ofn.lpstrFile)
  {
    theApp.free(ofn.lpstrFile);
    return false;
  }

  if(ofn.nFileOffset < strlen(ofn.lpstrFile))
  {
    if(createEditChild(ofn.lpstrFile)==-1)
    {
      theApp.free(ofn.lpstrFile);
      return false;
    }
  }
  else
  {
    DWORD dwLength=strlen(ofn.lpstrFile);
    LPSTR lpszName=ofn.lpstrFile+dwLength;
    LPSTR lpszPos=ofn.lpstrFile+ofn.nFileOffset;
    if(dwLength && *(lpszName-1)!='\\') *(lpszName++)='\\';

    while(*lpszPos)
    {
      dwLength=strlen(lpszPos)+1;
      memcpy(lpszName, lpszPos, dwLength);
      lpszPos+=dwLength;

      if(createEditChild(ofn.lpstrFile)==-1)
      {
        theApp.free(ofn.lpstrFile);
        return false;
      }
    }
  }

  theApp.free(ofn.lpstrFile);
  return true;
}

// -----------------------------------------------------------------------------------------------
// create new editchild window
int createEditChild(LPTSTR lpszName)
{
  //search a free slot
  for(int i=_MAX_OPEN_CHILDS; i--;) if( !pEditChild[i] )
  {
    if(!lpszName) //no file specified? show open dialog
    {
      showOpenDialog();
      return(-1);
    }

    pEditChild[i] = new EditChild(lpszName, i);
    if( !pEditChild[i]->fFailed ) return( i );  //creation error?
    if( pEditChild[i] ) delete pEditChild[i];
    return(-1);
  }

  // -> too much open windows
  theApp.doErrorDlg("too many open windows.");
  return(-1);
}

// -----------------------------------------------------------------------------------------------
// show current/selected editchild window
BOOL showEditChild(UINT uMsg, HWND hwndChild)
{
  if(!hwndChild)
    hwndChild = (HWND)SendMessage(theApp.hwndClient, WM_MDIGETACTIVE, NULL, NULL);

  if(hwndChild) switch(uMsg)
  {
  case EC_SHOW_ACTIVATE:
    SendMessage(theApp.hwndClient, WM_MDIACTIVATE, (WPARAM)hwndChild, NULL);
    return true;
  case EC_SHOW_RESTORE:
    SendMessage(theApp.hwndClient, WM_MDIRESTORE, (WPARAM)hwndChild, NULL);
    return true;
  case EC_SHOW_MINIMIZE:    
    ShowWindow(hwndChild, SW_MINIMIZE);
    return true;
  case EC_SHOW_MAXIMIZE:
    SendMessage(theApp.hwndClient, WM_MDIMAXIMIZE, (WPARAM)hwndChild, NULL);
    return true;
  }

  return false;
}

// -----------------------------------------------------------------------------------------------
// save current/selected editchild window
BOOL saveEditChild(WORD fwFlags, HWND hwndChild)
{
  if(!hwndChild)
    hwndChild = (HWND)SendMessage(theApp.hwndClient, WM_MDIGETACTIVE, NULL, NULL);

  if(hwndChild)
  {
    //to do: save this file
    //
    return true;
  }

  return false;
}

// -----------------------------------------------------------------------------------------------
// close current/selected editchild window
BOOL closeEditChild(HWND hwndChild)
{
  if(hwndChild == EC_CLOSE_ALL) // close all editchild window
  {
    for(int i=_MAX_OPEN_CHILDS; i--;) if(pEditChild[i])
      closeEditChild(pEditChild[i]->hwndEdit);

    return true;
  }

  if(!IsWindow(hwndChild))
    hwndChild = (HWND)SendMessage(theApp.hwndClient, WM_MDIGETACTIVE, NULL, NULL);

  if(IsWindow(hwndChild))
  {
    SendMessage(hwndChild, WM_CLOSE, 0, 0);
    return true;
  }

  return false;
}

// -----------------------------------------------------------------------------------------------
// stream into current edit controll (load text file)
DWORD CALLBACK streaminEditCntrl(HANDLE hFile, LPVOID lpBuff, LONG cb, LONG FAR *pcb)
{
  if(!ReadFile(hFile, lpBuff, cb, (DWORD *)pcb, NULL)) *pcb=0;
  return 0;
}

// -----------------------------------------------------------------------------------------------
// deinit
void deinitEditChild()
{
  //destroy loaded objects
  for(int i=_MAX_OPEN_CHILDS; i--;) if(pEditChild[i]) delete pEditChild[i];
}


// ==============================================================================================-
// the EditProc

LRESULT CALLBACK EditProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  int index = (int)GetWindowWord(hwnd, 0);
  
  if((index >= 0)&&(index < _MAX_OPEN_CHILDS)&&(pEditChild[index])
  &&(pEditChild[index]->hwndEdit == hwnd))
  {
    switch(uMsg)
    {
    case WM_SIZE:
      if(wParam == SIZE_MAXIMIZED) theApp.settings.fMaximizedEditChild=true;
      else theApp.settings.fMaximizedEditChild=false;

      MoveWindow(pEditChild[index]->hwndEditBar,
        0,0, _EDIT_COLUMN_SIZE_LEFT,HIWORD(lParam), true);
      MoveWindow(pEditChild[index]->hwndEditCntrl,
        _EDIT_COLUMN_SIZE_LEFT,0, LOWORD(lParam)-_EDIT_COLUMN_SIZE_LEFT,HIWORD(lParam), true);
      break;

    case WM_SETFOCUS:
      SetFocus( pEditChild[index]->hwndEditCntrl );
      break;

    case WM_CLOSE:
      //at this point: if changed, ask user to save file
      //
      delete pEditChild[index];
      return(0);

    case EN_ERRSPACE:
      theApp.doErrorDlg("out of memory.");
      break;
    }

    return( DefMDIChildProc(hwnd, uMsg, wParam, lParam) );
  }

  return( DefWindowProc(hwnd, uMsg, wParam, lParam) );
}


// ==============================================================================================-
// EditChild: methods


// -----------------------------------------------------------------------------------------------
// constructor
EditChild::EditChild(LPTSTR lpszName, int index)
{
  RECT rect;

  //init variables
  fFailed = false;
  hwndEdit = hwndEditBar = hwndEditCntrl = NULL;
  fChanged = false;
  lpszFileName = (LPSTR)theApp.malloc(strlen(lpszName)+1);
  if(!lpszFileName)
  {
  _failed:
    fFailed = true;
    theApp.doErrorDlg("create child window failed.");
    return;
  }
  strcpy(lpszFileName, lpszName);


  GetClientRect(theApp.hwndClient, &rect);
  if(pntNewWndPos.x > (rect.right/3*2) || pntNewWndPos.y > (rect.bottom/3*2))
    pntNewWndPos.x=pntNewWndPos.y=0;

  hwndEdit = CreateWindowEx(
    WS_EX_MDICHILD | WS_EX_WINDOWEDGE,
    (char *)theApp.atomEditClass,
    lpszName,
    WS_CHILD|WS_CLIPSIBLINGS|WS_BORDER|WS_DLGFRAME|WS_SYSMENU|WS_THICKFRAME|
    WS_OVERLAPPED|WS_MINIMIZEBOX|WS_MAXIMIZEBOX|0xC000,
    pntNewWndPos.x, pntNewWndPos.y,
    rect.right-pntNewWndPos.x, rect.bottom-pntNewWndPos.y,
    theApp.hwndClient,
    NULL,
    theApp.hInstance,
    NULL
  );
  /*MDICREATESTRUCT mdic;
  mdic.szClass = (char *)theApp.atomEditClass;
  mdic.szTitle = lpszName;
  mdic.hOwner = theApp.hwndClient;
  mdic.x = mdic.y = 16;
  mdic.cx = 320; mdic.cy = 200;
  mdic.style = 0;
  mdic.lParam = 0;
  hwndEdit = (HWND)SendMessage(theApp.hwndClient, WM_MDICREATE, 0, (LPARAM)&mdic);*/
  if( !hwndEdit ) goto _failed;

  SetWindowWord(hwndEdit, 0, (WORD)index);
  //SetClassLong(hwndEdit, GCL_HICON, (long)LoadIcon(NULL, 0)); // change class icon

  hwndEditBar = CreateWindowEx(
    0,
    (char *)theApp.atomEditBarClass,
    NULL,
    WS_CHILD | WS_CLIPSIBLINGS | WS_HSCROLL | WS_VISIBLE,
    0, 0,
    _EDIT_COLUMN_SIZE_LEFT, 0,
    hwndEdit,
    NULL,
    theApp.hInstance,
    NULL
  );

  if( !hwndEditBar ) goto _failed;
  SetWindowWord(hwndEditBar, 0, (WORD)-1);


  hwndEditCntrl = CreateWindowEx(
    0,
    "RichEdit",
    NULL,
    WS_CHILD | WS_CLIPSIBLINGS | WS_VSCROLL | WS_HSCROLL | WS_VISIBLE |
    ES_MULTILINE | ES_AUTOVSCROLL | ES_AUTOHSCROLL | ES_LEFT | ES_SAVESEL | ES_DISABLENOSCROLL,
    0, 0,
    0, 0,
    hwndEdit,
    NULL,
    theApp.hInstance,
    NULL
  );
  if( !hwndEditCntrl ) goto _failed;
  SendMessage(hwndEditCntrl, EM_SETLIMITTEXT, 65536*10, 0); //64k is default
  SendMessage(hwndEditCntrl, EM_SETREADONLY, true, 0); //lock
  //SendMessage(hwndEditCntrl, EM_SETMARGINS, EC_LEFTMARGIN, MAKELONG(2,0));


  //load file now...
  HANDLE hFile = CreateFile(lpszName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
  if(hFile != INVALID_HANDLE_VALUE)
  {
    EDITSTREAM es;
    es.dwCookie = (DWORD)hFile;
    es.dwError = 0;
    es.pfnCallback = (EDITSTREAMCALLBACK)streaminEditCntrl;
    SendMessage(hwndEditCntrl, EM_STREAMIN, SF_TEXT, (LPARAM)&es);
    CloseHandle(hFile);
  }

  CHARFORMAT cf;
  cf.cbSize = sizeof(CHARFORMAT);
  cf.dwMask = CFM_FACE|CFM_SIZE|CFM_BOLD;
  cf.dwEffects = 0;
  cf.yHeight = 10;
  sprintf(cf.szFaceName, "Courier");
  SendMessage(hwndEditCntrl, EM_SETCHARFORMAT, SCF_ALL, (LPARAM)&cf);

  SendMessage(hwndEditCntrl, EM_SETREADONLY, false, 0); //unlock


  // show window
  if(theApp.settings.fMaximizedEditChild)
    PostMessage(theApp.hwndClient, WM_MDIMAXIMIZE, (WPARAM)hwndEdit, NULL);
  PostMessage(theApp.hwndClient, WM_MDIACTIVATE, (WPARAM)hwndEdit, NULL);
  PostMessage(theApp.hwndClient, WM_MDIREFRESHMENU, 0, 0);

  GetClientRect(hwndEdit, &rect);
  MoveWindow(hwndEditBar, 0,0,
    _EDIT_COLUMN_SIZE_LEFT,rect.bottom, true);
  MoveWindow(hwndEditCntrl,
    _EDIT_COLUMN_SIZE_LEFT,0,
    rect.right-_EDIT_COLUMN_SIZE_LEFT,rect.bottom, true);

  ShowWindow(hwndEdit, SW_SHOWDEFAULT);

  // precalc pos of next edit window
  pntNewWndPos.x+=20; pntNewWndPos.y+=20;
}

// -----------------------------------------------------------------------------------------------
// destructor
EditChild::~EditChild()
{
  SendMessage(theApp.hwndClient, WM_MDIDESTROY, (WPARAM)hwndEdit, NULL);
  //PostMessage(theApp.hwndClient, WM_MDIREFRESHMENU, 0, 0);

  if(IsWindow(hwndEdit)) DestroyWindow(hwndEdit);
  if(IsWindow(hwndEditBar)) DestroyWindow(hwndEditBar);
  if(IsWindow(hwndEditCntrl)) DestroyWindow(hwndEditCntrl);

  if(lpszFileName) theApp.free(lpszFileName);

  for(int i=_MAX_OPEN_CHILDS; i--;) if(pEditChild[i] == this)
  {
    pEditChild[i] = NULL;
    break;
  }
}


// -----------------------------------------------------------------------------------------------
