// ==============================================================================================-
// MainApp.cpp : class for the main application window
// ==============================================================================================-

#include "main.h"
#include "MainApp.h"
#include "EditChild.h"
#include "Workspace.h"
#include "OutputWnd.h"


// ==============================================================================================-
// constants

TBBUTTON tbbToolbarStruct[_NUM_TOOLBAR_BTNS] =
{
  {0, ID_FILE_NEW,   TBSTATE_ENABLED,       TBSTYLE_BUTTON, 0, 0},
  {1, ID_FILE_OPEN,  TBSTATE_ENABLED,       TBSTYLE_BUTTON, 0, 0},
  {2, ID_FILE_SAVE,  TBSTATE_INDETERMINATE, TBSTYLE_BUTTON, 0, 0},
  {0, 0,             TBSTATE_ENABLED,       TBSTYLE_SEP,    0, 0},
  {3, ID_EDIT_CUT,   TBSTATE_INDETERMINATE, TBSTYLE_BUTTON, 0, 0},
  {4, ID_EDIT_COPY,  TBSTATE_INDETERMINATE, TBSTYLE_BUTTON, 0, 0},
  {5, ID_EDIT_PASTE, TBSTATE_INDETERMINATE, TBSTYLE_BUTTON, 0, 0},
  {0, 0,             TBSTATE_ENABLED,       TBSTYLE_SEP,    0, 0},
  {6, ID_FILE_PRINT, TBSTATE_INDETERMINATE, TBSTYLE_BUTTON, 0, 0},
  {7, ID_APP_ABOUT,  TBSTATE_ENABLED,       TBSTYLE_BUTTON, 0, 0},
};

#ifdef _DEBUG
  DWORD dwMallocCntr = 0;
#endif


// ==============================================================================================-
// global functions

// -----------------------------------------------------------------------------------------------
BOOL AboutDlgProc(HWND hdlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch(uMsg)
  {
  case WM_INITDIALOG:
    return true;

  case WM_COMMAND:    
    switch(LOWORD(wParam))
    {
    case IDOK:
      EndDialog(hdlg, 0);
    }
    break;

  case WM_CLOSE:
  case WM_DESTROY:
    EndDialog(hdlg, 0);
    return true;
  }

  return false;
}



// ==============================================================================================-
// MainApp: methods


// -----------------------------------------------------------------------------------------------
// constructs a MainApp object
MainApp::MainApp(LPTSTR lpszName)
{
  //init variables
  lpszAppName = lpszName;

  hInstance = NULL;
  lpszCmdLine = NULL;
  nCmdShow = NULL;
  dwClassCntr = NULL;

  atomMainClass =
  atomWorkspaceClass =
  atomOutputClass =
  atomEditClass =
  atomEditBarClass = NULL;

  hmenuMain = NULL;

  hwndMain =
  hwndClient =
  hwndToolBar =
  hwndStatusBar =
  hwndWorkspace =
  hwndOutput = NULL;

  lpszExePath =
  lpszHelpFilePath =
  lpszProfileName =
  lpszRegistryKey = NULL;
}

// -----------------------------------------------------------------------------------------------
// destructs
MainApp::~MainApp()
{
  #ifdef _DEBUG
    if(dwMallocCntr) MessageBox(NULL, "malloc counter not zero!", "debug error", MB_OK|MB_ICONERROR);
  #endif
}


// -----------------------------------------------------------------------------------------------
// Windows instance initialization
// (creating window objects and loads profile settings or calls FirstRun)
BOOL MainApp::initInstance()
{
  //init
  initEditChild();

  // init registry access (on errors do NOT set lpszRegistryKey value)
  //...
  //SetRegistryKey(_T("MyApp"));
  //lpszRegistryKey = "xxx";

  // init INI file and load app profile settings
  //...
  // on error set values to default (firstRun)
  if(!firstRun()) return false;


  //load menu
  hmenuMain = LoadMenu(hInstance, MAKEINTRESOURCE(IDR_MENU));


  // register class
  WNDCLASS wc;

  wc.style = CS_DBLCLKS | CS_BYTEALIGNCLIENT | CS_BYTEALIGNWINDOW;
  wc.lpfnWndProc = &WndProc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = hInstance;
  wc.hIcon = loadIconResource(IDI_MAIN);
  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = NULL;
  wc.lpszMenuName = NULL;
  if( !(atomMainClass = registerClass(&wc)) ) return false;

  wc.hIcon = NULL;
  wc.cbWndExtra = 4;
  wc.hbrBackground = (HBRUSH)(COLOR_INACTIVEBORDER+1);
  wc.lpfnWndProc = &WorkspaceWndProc;
  if( !(atomWorkspaceClass = registerClass(&wc)) ) return false;

  wc.lpfnWndProc = &OutputWndProc;
  if( !(atomOutputClass = registerClass(&wc)) ) return false;

  wc.lpfnWndProc = &EditProc;
  wc.hIcon = loadIconResource(IDI_EDIT_DEF);
  wc.hbrBackground = NULL;
  if( !(atomEditClass = registerClass(&wc)) ) return false;

  // create EditBarBrush for EditBarClass
  struct {
   BITMAPINFOHEADER head; 
   RGBQUAD rgb[2];
   DWORD dwBits[8];
  } bmi;

  bmi.head.biSize = sizeof(BITMAPINFOHEADER);
  bmi.head.biWidth = 8;
  bmi.head.biHeight = 8;
  bmi.head.biPlanes = 1;
  bmi.head.biBitCount = 1;
  bmi.head.biCompression = BI_RGB;
  bmi.head.biSizeImage = 8*4;
  bmi.head.biXPelsPerMeter = 0;
  bmi.head.biYPelsPerMeter = 0;
  bmi.head.biClrUsed = 0;
  bmi.head.biClrImportant = 0;

  #define RGBc(n,c) \
    bmi.rgb[n].rgbRed   = (BYTE)(GetSysColor(c)>>0)&255;\
    bmi.rgb[n].rgbGreen = (BYTE)(GetSysColor(c)>>8)&255;\
    bmi.rgb[n].rgbBlue  = (BYTE)(GetSysColor(c)>>16)&255;

  RGBc(0, COLOR_3DHILIGHT)
  RGBc(1, COLOR_3DLIGHT)
  #undef RGBc

  bmi.dwBits[0]=0xAA; bmi.dwBits[1]=0x55;
  bmi.dwBits[2]=0xAA; bmi.dwBits[3]=0x55;
  bmi.dwBits[4]=0xAA; bmi.dwBits[5]=0x55;
  bmi.dwBits[6]=0xAA; bmi.dwBits[7]=0x55;

  wc.hbrBackground = CreateDIBPatternBrushPt((void*)&bmi, DIB_RGB_COLORS);
  if(!wc.hbrBackground) wc.hbrBackground = (HBRUSH)(COLOR_INACTIVEBORDER+1);

  wc.hIcon = NULL;
  wc.hCursor = LoadCursor(NULL, IDC_NO);  
  if( !(atomEditBarClass = registerClass(&wc)) ) return false;

  // create main MDI window
  hwndMain = CreateWindowEx(
    WS_EX_ACCEPTFILES|WS_EX_WINDOWEDGE|WS_EX_LEFT, // | WS_EX_CONTEXTHELP 
    (char *)atomMainClass, lpszAppName,
    WS_CLIPSIBLINGS|WS_MAXIMIZE|WS_BORDER|WS_DLGFRAME|WS_SYSMENU|WS_THICKFRAME|
    WS_OVERLAPPED|WS_MINIMIZEBOX|WS_MAXIMIZEBOX|0x8000,
    CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
    NULL, hmenuMain, hInstance, NULL
  );
  if( !hwndMain ) return false;


  // create client window
  CLIENTCREATESTRUCT ccs;
  
  ccs.hWindowMenu = GetSubMenu(hmenuMain, 6);
  ccs.idFirstChild = ID_WINDOW_LIST;

  hwndClient = CreateWindowEx(
    WS_EX_ACCEPTFILES|WS_EX_CLIENTEDGE, "MDICLIENT", NULL,
    WS_CHILD|WS_CLIPSIBLINGS|WS_CLIPCHILDREN|WS_OVERLAPPED|WS_VISIBLE|0x01,
    0, 0, 0, 0, hwndMain, hmenuMain, hInstance, &ccs
  );
  if( !hwndClient ) return false;


  // create toolbar
  hwndToolBar = CreateToolbarEx(
    hwndMain, WS_CHILD, 2000,
    _NUM_TOOLBAR_BTNS, hInstance, IDR_TOOLBAR,
    &tbbToolbarStruct[0], _NUM_TOOLBAR_BTNS,
    16, 16, 16, 16, sizeof( TBBUTTON )
  );
  if( !hwndToolBar ) return false;
  
  CheckMenuItem(hmenuMain, ID_VIEW_TOOLBAR, (settings.fViewToolBar)?MF_CHECKED:MF_UNCHECKED);
  if(settings.fViewToolBar) ShowWindow(hwndToolBar, SW_SHOW);
  else EnableWindow(hwndToolBar, false);


  // create statusbar
  hwndStatusBar = CreateStatusWindow(WS_CHILD, NULL, hwndMain, 1000);
  if( !hwndStatusBar ) return false;

  CheckMenuItem(hmenuMain, ID_VIEW_STATUSBAR, (settings.fViewStatusBar)?MF_CHECKED:MF_UNCHECKED);
  if(settings.fViewStatusBar) ShowWindow(hwndStatusBar, SW_SHOW);
  else EnableWindow(hwndStatusBar, false);
  
  // create workspace window
  hwndWorkspace = CreateWindowEx(
    WS_EX_ACCEPTFILES, (char *)atomWorkspaceClass, NULL,
    WS_CHILD|WS_CLIPCHILDREN|WS_OVERLAPPED,
    0, 0, 0, 0, hwndMain, NULL, hInstance, NULL
  );
  if(!hwndWorkspace) return false;
  
  CheckMenuItem(hmenuMain, ID_VIEW_WORKSPACE, (settings.fViewWorkspace)?MF_CHECKED:MF_UNCHECKED);
  if(settings.fViewWorkspace) ShowWindow(hwndWorkspace, SW_SHOW);
  else EnableWindow(hwndWorkspace, false);


  // create output window
  hwndOutput = CreateWindowEx(
    WS_EX_ACCEPTFILES, (char *)atomOutputClass, NULL,
    WS_CHILD|WS_CLIPCHILDREN|WS_OVERLAPPED,
    0, 0, 0, 0, hwndMain, NULL, hInstance, NULL
  );
  if(!hwndOutput) return false;

  CheckMenuItem(hmenuMain, ID_VIEW_OUTPUT, (settings.fViewOutputWnd)?MF_CHECKED:MF_UNCHECKED);
  if(settings.fViewOutputWnd) ShowWindow(hwndOutput, SW_SHOW);
  else EnableWindow(hwndOutput, false);


  // parse command line for DDE (file open commands)
  //...

  // show main window
  ShowWindow( hwndMain, nCmdShow );
  //UpdateWindow( hwndMain );

  return true;
}

// -----------------------------------------------------------------------------------------------
// clean up when your application terminates
void MainApp::exitInstance()
{
  if(IsWindow(hwndMain)) DestroyWindow(hwndMain);
  if(IsWindow(hwndClient)) DestroyWindow(hwndClient);
  if(IsWindow(hwndToolBar)) DestroyWindow(hwndToolBar);
  if(IsWindow(hwndStatusBar)) DestroyWindow(hwndStatusBar);
  if(IsWindow(hwndWorkspace)) DestroyWindow(hwndWorkspace);
  if(IsWindow(hwndOutput)) DestroyWindow(hwndOutput);

  deinitEditChild();

  if(hmenuMain) DestroyMenu(hmenuMain);
  if(lpszExePath) free(lpszExePath);
  if(lpszHelpFilePath) free(lpszHelpFilePath);
  if(lpszProfileName) free(lpszProfileName);
  if(lpszRegistryKey) free(lpszRegistryKey);
}

// -----------------------------------------------------------------------------------------------
// close edit-windows (if possible) and write profile settings (return true for termination)
BOOL MainApp::exitApplication()
{
  closeEditChild(EC_CLOSE_ALL);
  return true;
}


// -----------------------------------------------------------------------------------------------
// is executed if the app runs the first time
BOOL MainApp::firstRun()
{
  settings.fViewWorkspace = true;
  settings.fViewOutputWnd = true;
  settings.fViewToolBar = true;
  settings.fViewStatusBar = true;

  settings.fMaximizedEditChild = true;

  settings.nWorkspaceWidth = 140;
  settings.nOutputWndHeight = 120;

  return true;
}


// -----------------------------------------------------------------------------------------------
// runs the default message loop
int MainApp::run()
{
  MSG lpMsg;
  int nRet;


  //load accelerators from resource
  HACCEL hAccTable = LoadAccelerators(hInstance, MAKEINTRESOURCE(IDR_ACCELERATOR));

  //run message loop
  for(;;)
  {
    nRet = GetMessage(&lpMsg, NULL, 0, 0);
    switch(nRet)
    {
    case -1:  //error
      doErrorDlg("error within message loop.");
      return( 0 );

    case 0:   //WM_QUIT
      //exitInstance();
      return( lpMsg.wParam );

    default:  //WM_XXX
      if( !TranslateAccelerator(hwndMain, hAccTable, &lpMsg) )
      if( !TranslateMDISysAccel(hwndClient, &lpMsg) )
      //if( !IsDialogMessage(lpMsg.hwnd, &lpMsg) )
      { 
        TranslateMessage(&lpMsg);
        DispatchMessage(&lpMsg);
      }
    }

  }

  //return( 0 );
}


// -----------------------------------------------------------------------------------------------
// handle menu commands
void MainApp::executeCommand(WORD wCmd)
{
  switch(wCmd)
  {
  //system menu (icon of maximized child window)
  case SC_RESTORE:
    showEditChild(EC_SHOW_RESTORE);
    break;

  case SC_MINIMIZE:
    showEditChild(EC_SHOW_MINIMIZE);
    break;

  //file menu
  case ID_FILE_NEW:
    createEditChild("Noname");  // test editchild window
    break;
  case ID_FILE_OPEN:
    //open dialog box...
    createEditChild();
    break;
  case ID_FILE_CLOSE:
    closeEditChild();
    break;
  case ID_FILE_SAVE:
    saveEditChild(EC_SAVE_DEFAULT);
    break;
  case ID_FILE_SAVE_AS:
    saveEditChild(EC_SAVE_AS);
    break;
  case ID_FILE_SAVE_ALL:
    saveEditChild(EC_SAVE_ALL);
    break;


  //view menu
  case ID_VIEW_WORKSPACE:
    settings.fViewWorkspace ^= 1;
    CheckMenuItem(hmenuMain, ID_VIEW_WORKSPACE, (settings.fViewWorkspace)?MF_CHECKED:MF_UNCHECKED);
    ShowWindow(hwndWorkspace, (settings.fViewWorkspace)?SW_SHOW:SW_HIDE);
    EnableWindow(hwndWorkspace, settings.fViewWorkspace);
    onSize();
    break;
    
  case ID_VIEW_OUTPUT:
    settings.fViewOutputWnd ^= 1;
    CheckMenuItem(hmenuMain, ID_VIEW_OUTPUT, (settings.fViewOutputWnd)?MF_CHECKED:MF_UNCHECKED);
    ShowWindow(hwndOutput, (settings.fViewOutputWnd)?SW_SHOW:SW_HIDE);
    EnableWindow(hwndOutput, settings.fViewOutputWnd);
    onSize();
    break;

  case ID_VIEW_TOOLBAR:
    settings.fViewToolBar ^= 1;
    CheckMenuItem(hmenuMain, ID_VIEW_TOOLBAR, (settings.fViewToolBar)?MF_CHECKED:MF_UNCHECKED);
    ShowWindow(hwndToolBar, (settings.fViewToolBar)?SW_SHOW:SW_HIDE);
    EnableWindow(hwndToolBar, settings.fViewToolBar);
    onSize();
    break;

  case ID_VIEW_STATUSBAR:
    settings.fViewStatusBar ^= 1;
    CheckMenuItem(hmenuMain, ID_VIEW_STATUSBAR, (settings.fViewStatusBar)?MF_CHECKED:MF_UNCHECKED);
    ShowWindow(hwndStatusBar, (settings.fViewStatusBar)?SW_SHOW:SW_HIDE);
    EnableWindow(hwndStatusBar, settings.fViewStatusBar);
    onSize();
    break;


  //menu window
  case ID_WINDOW_NEW:
    createEditChild("window->new");
    break;
  case SC_CLOSE:
  case ID_WINDOW_CLOSE:
    closeEditChild();
    break;
  case ID_WINDOW_CLOSE_ALL:
    closeEditChild(EC_CLOSE_ALL);
    break;
  case ID_WINDOW_PREV:
    SendMessage(hwndClient, WM_MDINEXT, NULL, 1);
    break;
  case SC_NEXTWINDOW:
  case ID_WINDOW_NEXT:
    SendMessage(hwndClient, WM_MDINEXT, NULL, 0);
    break;
  case ID_WINDOW_CASCADE:
    SendMessage(hwndClient, WM_MDICASCADE, NULL, NULL);
    break;
  case ID_WINDOW_TILE_HORZ:
    SendMessage(hwndClient, WM_MDITILE, MDITILE_HORIZONTAL, NULL);
    break;
  case ID_WINDOW_TILE_VERT:
    SendMessage(hwndClient, WM_MDITILE, MDITILE_VERTICAL, NULL);
    break;
  case ID_WINDOW_ARRANGE:
    SendMessage(hwndClient, WM_MDIICONARRANGE, NULL, NULL);
    break;

  //application
  case ID_APP_EXIT:
    SendMessage(hwndMain, WM_CLOSE, NULL, NULL);
    break;
  case ID_APP_ABOUT:
    DialogBox(hInstance, MAKEINTRESOURCE(IDD_ABOUTBOX), hwndMain, (DLGPROC)AboutDlgProc);
    break;

#ifdef _DEBUG
  default:
    char msg[80];
    sprintf(msg, "not supported function: %08X", LOWORD(wCmd));
    MessageBox(NULL, msg, NULL, 0);
#endif

  }
}


// -----------------------------------------------------------------------------------------------
// updates statusbar-text
void MainApp::onMenuSelect(BOOL fOpen, UINT uSelect)
{
  if(fOpen) //open menu now
  {
    //updateMainMenu(); reset all menu-items
    SendMessage(hwndStatusBar, SB_SETTEXT, 0, 0);
  }
  else
  {
    char szStatusText[64];
    LoadString(hInstance, uSelect?uSelect:IDS_READY, szStatusText, 64);
    SendMessage(hwndStatusBar, SB_SETTEXT, 0, (LPARAM)szStatusText);
  }
}

// -----------------------------------------------------------------------------------------------
// handles the windows messages
LRESULT MainApp::MessageHandler(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch(uMsg)
  {
  case WM_SIZE:
    if(hwnd == hwndMain) return onSize();
    break;


  case WM_MENUSELECT:
    onMenuSelect((HMENU)lParam == hmenuMain, LOWORD(wParam));
    break;

  case WM_COMMAND:
    executeCommand(LOWORD(wParam));
    break;


  case WM_CLOSE:    
    if(hwnd != hwndMain) break;
    if(exitApplication()) exitInstance();
    return(0);

  case WM_DESTROY:
    PostQuitMessage(NULL);
    return false;

//  case WM_USER:
//    return false;
  }

  return( DefWindowProc(hwnd, uMsg, wParam, lParam) );
}


// -----------------------------------------------------------------------------------------------
// called when window is resized (also minimized)
LRESULT MainApp::onSize(void)
{
  RECT rect;
  int v1,v2,v3;
  int h4;

  if(settings.fViewWorkspace)
  {
    h4 = settings.nWorkspaceWidth;
    if(h4 < WORKSPACE_WIDTH_MIN+WORKSPACE_SPACE_L+WORKSPACE_SPACE_R)
      h4 = WORKSPACE_WIDTH_MIN+WORKSPACE_SPACE_L+WORKSPACE_SPACE_R;
    GetClientRect(hwndMain, &rect);
    if(h4 > rect.right-WORKSPACE_WIDTH_MIN) h4 = rect.right-WORKSPACE_WIDTH_MIN;
  }
  else h4 = 0;

  if(settings.fViewOutputWnd)
  {
    v3 = settings.nOutputWndHeight;
    if(v3 < OUTPUT_HEIGHT_MIN+OUTPUT_SPACE_T+OUTPUT_SPACE_B)
      v3 = OUTPUT_HEIGHT_MIN+OUTPUT_SPACE_T+OUTPUT_SPACE_B;
    GetClientRect(hwndMain, &rect); v1 = rect.bottom;
    GetClientRect(hwndToolBar, &rect); v1-=rect.bottom;
    v1-=GetSystemMetrics(SM_CYMENUSIZE);
    if(v3 > v1-OUTPUT_HEIGHT_MIN) v3 = v1-OUTPUT_HEIGHT_MIN;
  }
  else v3 = 0;

  if(settings.fViewStatusBar)
  {GetWindowRect(hwndStatusBar, &rect); v2 = (int)(rect.bottom-rect.top);}
  else v2 = 0;

  if(settings.fViewToolBar)
  {GetWindowRect(hwndToolBar, &rect); v1 = (int)(rect.bottom-rect.top);}
  else v1 = 0;


  GetClientRect(hwndMain, &rect);
  #define nWidth rect.right
  #define nHeight rect.bottom

  SetWindowPos(hwndClient, HWND_TOP, h4, v1, nWidth-h4, nHeight-v1-v2-v3,
    SWP_NOACTIVATE);

  if(settings.fViewOutputWnd)
    SetWindowPos(hwndOutput, HWND_TOP, 0, nHeight-v2-v3, nWidth, v3, SWP_NOACTIVATE);
  if(settings.fViewWorkspace)
    SetWindowPos(hwndWorkspace, HWND_TOP, 0, v1, h4, nHeight-v1-v2-v3, SWP_NOACTIVATE);
  
  if(settings.fViewStatusBar)
    SetWindowPos(hwndStatusBar, HWND_TOP, 0, nHeight-v2, nWidth, v2, SWP_NOACTIVATE);
  if(settings.fViewToolBar)
    SetWindowPos(hwndToolBar, HWND_TOP, 0, 0, nWidth, v1, SWP_NOACTIVATE);

  //RedrawWindow(hwndClient, NULL, NULL, RDW_INVALIDATE);
  #undef nHeight
  #undef nWidth

  return(1);
}


// -----------------------------------------------------------------------------------------------
// registers a new class with unique name
ATOM MainApp::registerClass(WNDCLASS *lpwc)
{
  LPTSTR lpszBuf = (LPTSTR)LocalAlloc(LPTR, strlen(lpszAppName) + 16);
  if(!lpszBuf) return(NULL);

  sprintf(lpszBuf, "%s [%08X]", lpszAppName, dwClassCntr++);
  lpwc->lpszClassName = lpszBuf;

  ATOM atom = RegisterClass(lpwc);
  LocalFree(lpszBuf);
  return(atom);
}


// -----------------------------------------------------------------------------------------------
// shows an error dialog
void MainApp::doErrorDlg(LPTSTR lpszText)
{
  if( lpszText )  //just do output
  {
    MessageBox(hwndMain, lpszText, lpszAppName, MB_OK | MB_ICONERROR);
  }
  else      //load error string from sytem
  {
    DWORD dwErr = GetLastError();
    if( !dwErr ) return;
    if( FormatMessage(
        FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
        NULL, dwErr, 0, (LPTSTR)&lpszText, 0, NULL) )
    {
      MessageBox(hwndMain, lpszText, lpszAppName, MB_OK | MB_ICONERROR);
      LocalFree(lpszText);
    }
    else
    {
      doErrorDlg("create error message failed.");
    }
  }
}


// -----------------------------------------------------------------------------------------------
// loads an icon resource
HICON MainApp::loadIconResource(WORD uID)
{
  HICON h = LoadIcon(hInstance, MAKEINTRESOURCE(uID));
  if( !h ) doErrorDlg();
  return( h );
}

// -----------------------------------------------------------------------------------------------
// loads a cursor resource
HCURSOR MainApp::loadCursorResource(WORD uID)
{
  HCURSOR h = LoadCursor(hInstance, MAKEINTRESOURCE(uID));
  if( !h ) doErrorDlg();
  return( h );
}

// -----------------------------------------------------------------------------------------------
// loads a string resource
LPTSTR MainApp::loadStringResource(WORD uID, WORD uSize)
{
  LPTSTR lpsz = (LPTSTR)LocalAlloc(LPTR, uSize);
  if( !lpsz ) doErrorDlg();
  else
  {

    if( LoadString(hInstance, uID, lpsz, uSize) ) return( lpsz );
    else
    {
      doErrorDlg();
      LocalFree(lpsz);
    }
  }

  return( NULL );
}

// -----------------------------------------------------------------------------------------------
// alloc memory
void *MainApp::malloc(DWORD dwSize)
{
  void *ptr;
  if( !(ptr = GlobalAlloc(GPTR, dwSize)) ) doErrorDlg();
  #ifdef _DEBUG
    if(dwMallocCntr!=(DWORD)-1) dwMallocCntr++;
  #endif
  return( ptr );
}

// -----------------------------------------------------------------------------------------------
// free memory
void MainApp::free(void *lpHndl)
{
  if(!lpHndl) return;
  GlobalFree(lpHndl);
  #ifdef _DEBUG
    if(dwMallocCntr!=(DWORD)-1) dwMallocCntr--;
  #endif
}