// ==============================================================================================-
// MainApp.h : header file for MainApp class
// ==============================================================================================-

#if !defined(_MAINAPP_H_)
#define _MAINAPP_H_


// ==============================================================================================-
// definitions
//

#define _NUM_TOOLBAR_BTNS 10
#define ID_WINDOW_LIST 0x8080

typedef struct
{
  
  BOOL  fViewWorkspace;
  BOOL  fViewOutputWnd;
  BOOL  fViewToolBar;
  BOOL  fViewStatusBar;

  BOOL  fMaximizedEditChild;

  int   nWorkspaceWidth;
  int   nOutputWndHeight;

} SETTINGS;


// ==============================================================================================-
// class MainApp : implemented in main.cpp
//

class MainApp
{
// -----------------------------------------------------------------------------------------------
// methods

public:
  MainApp(LPTSTR lpszName); // constructs a MainApp object
  ~MainApp();               // destructs

  BOOL  initInstance();     // Windows instance initialization (creating window objects and loads profile settings or calls FirstRun)
  BOOL  firstRun();         // is executed if the app runs the first time
  int   run();              // runs the default message loop
  void  exitInstance();     // clean up when your application terminates
  BOOL  exitApplication();  // close edit-windows (if possible) and write profile settings

  void  executeCommand(WORD wCmd);  //receives menu commands
  void  onMenuSelect(BOOL fOpen, UINT uSelect);

  LRESULT MessageHandler(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);



//Operations
  void  updateWindow();     // updates the entire main window

  ATOM  registerClass(WNDCLASS *lpwc);      // registers a new class with unique name
  void  doErrorDlg(LPTSTR lpszText = NULL); // shows an error dialog

  HICON loadIconResource(WORD uID);     // loads an icon resource
  HCURSOR loadCursorResource(WORD uID); // loads a cursor resource
  LPTSTR  loadStringResource(WORD uID, WORD uSize = 256); // loads a string resource

  void *malloc(DWORD dwSize);
  void  free(void *lpHndl);


  //AddDocTemplate      // adds a document template to the application's list of available document templates
  //GetFirstDocTemplatePosition // retrieves the position of the first document template
  //GetNextDocTemplate  // retrieves the position of a document template. Can be used recursively
  //OpenDocumentFile    // called by the framework to open a document from a file
  //CloseAllDocuments   // closes all open documents
  //SaveAllModifiedDocs // prompts the user to save all modified documents
  //SelectDocPrinter    // selects a printer previously indicated by a user through a print dialog box
  //CreatePrinterDC     // creates a printer device context
  //GetPrinterDeviceDefaults  // retrieves the printer device defaults
 
  //AddToRecentFileList // adds a filename to the most recently used (MRU) file list


  //void hideApplication();   // hides the application before closing all documents
  //DoWaitCursor        // turns the wait cursor on and off
  //WinHelp             // calls the WinHelp Windows function

  //ProcessWndProcException // intercepts all unhandled exceptions thrown by the application's message and command handlers


//Command Handlers
  LRESULT onSize(void);

  //BOOL onIdle(long lCount); // application-specific idle-time processing
  //OnDDECommand        // called on a dynamic data exchange (DDE) execute command

  //OnFileNew           // implements the ID_FILE_NEW command
  //OnFileOpen          // implements the ID_FILE_OPEN command
  //OnFilePrintSetup    // implements the ID_FILE_PRINT_SETUP command
  //OnContextHelp       // handles SHIFT+F1 Help within the application
  //OnHelp              // handles F1 Help within the application (using the current context)
  //OnHelpIndex         // handles the ID_HELP_INDEX command and provides a default Help topic
  //OnHelpFinder        // handles the ID_HELP_FINDER and ID_DEFAULT_HELP commands
  //OnHelpUsing         // handles the ID_HELP_USING command


// -----------------------------------------------------------------------------------------------
// data members

public:
  //initialized by constructor
  LPTSTR lpszAppName;   // specifies the name of the application

  //initialized by WinMain
  HINSTANCE hInstance;  // identifies the current instance of the application
  LPTSTR lpszCmdLine;   // points to a null-terminated command line string
  int nCmdShow;         // specifies how the window is to be shown initially

  //initialized by InitInstance
  DWORD dwClassCntr;    // counts the registerd classes (give unique names)
  ATOM atomMainClass;   // the class atoms...
  ATOM atomWorkspaceClass;
  ATOM atomOutputClass;
  ATOM atomEditClass;
  ATOM atomEditBarClass;

  HMENU hmenuMain;      // handle to main menu

  HWND hwndMain;        // handle of the main window
  HWND hwndClient;      // the client window holding (parent of the edit windows)
  HWND hwndToolBar;     // handle of the toolbar window
  HWND hwndStatusBar;   // handle of the statusbar window
  HWND hwndWorkspace;
  HWND hwndOutput;

  LPSTR lpszExePath;      // the path of the module file
  LPSTR lpszHelpFilePath; // the path to the application's Help file
  LPSTR lpszProfileName;  // the application's .INI filename
  LPSTR lpszRegistryKey;  // the full registry key for storing application profile settings (or NULL if not)

  //others
  SETTINGS settings;
};

//
// ==============================================================================================-


#endif