// ==============================================================================================-
// Workspace.cpp : class for the wndow on the left
// ==============================================================================================-

#include "main.h"
#include "OutputWnd.h"


// ==============================================================================================-
// global data

// ==============================================================================================-
// global functions

LRESULT CALLBACK OutputWndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  OutputWnd *lpWs;
  switch(uMsg)
  {
  case WM_CREATE:
    lpWs = new OutputWnd(hwnd);
    if(lpWs->fFailed) return(-1);
    SetWindowLong(hwnd, 0, (LONG)lpWs);
    break;

  case WM_MOUSEMOVE:
    lpWs = (OutputWnd*)GetWindowLong(hwnd, 0);
    if(lpWs && lpWs->OnMouseMove(hwnd)) return(0);
    break;

  case WM_SETCURSOR:
    lpWs = (OutputWnd*)GetWindowLong(hwnd, 0);
    if(lpWs && lpWs->OnSetCursor(hwnd)) return(0);
    break;

  case WM_SIZE:
    lpWs = (OutputWnd*)GetWindowLong(hwnd, 0);
    if(lpWs && lpWs->OnSize(hwnd, lParam)) return(0);
    break;

  case WM_LBUTTONUP:
    lpWs = (OutputWnd*)GetWindowLong(hwnd, 0);
    if(lpWs && lpWs->SetDragState(hwnd, false)) return(0);
    break;
  case WM_LBUTTONDOWN:
    lpWs = (OutputWnd*)GetWindowLong(hwnd, 0);
    if(lpWs && lpWs->SetDragState(hwnd, true)) return(0);
    break;

  case WM_DESTROY:
    lpWs = (OutputWnd*)GetWindowLong(hwnd, 0);
    if(lpWs) delete lpWs;
    break;
  }

  return( DefWindowProc(hwnd, uMsg, wParam, lParam) );
}


// ==============================================================================================-
// OutputWnd: methods


// -----------------------------------------------------------------------------------------------
// constructor
OutputWnd::OutputWnd(HWND hwnd)
{
  fFailed = false;
  hwndParent = hwnd;

  bDragState = 0;
  hCursor = LoadCursor(NULL, IDC_SIZENS);

  hwndTree = CreateWindowEx(
		WS_EX_CLIENTEDGE, //|WS_EX_ACCEPTFILES,
		WC_TREEVIEW, NULL,
		WS_VISIBLE|WS_CHILD|
    TVS_SHOWSELALWAYS|TVS_HASLINES|TVS_HASBUTTONS|TVS_LINESATROOT,
		0,0,0,0, hwnd, NULL, theApp.hInstance, NULL
	);
  if(!hwndTree)
  {
  //_failed:
    fFailed = true;
    theApp.doErrorDlg("create output window failed.");
    return;
  }


  TV_INSERTSTRUCT tvi;
  tvi.hParent = TVI_ROOT;
  tvi.hInsertAfter = TVI_LAST;
  tvi.item.mask = TVIF_TEXT;
  tvi.item.pszText = "no output available";
  TreeView_InsertItem(hwndTree, &tvi);

  //EnableWindow(hwndTree, false);
}

// -----------------------------------------------------------------------------------------------
// destructor
OutputWnd::~OutputWnd()
{
}


// -----------------------------------------------------------------------------------------------
// called on WM_LBUTTONXXX
BOOL OutputWnd::SetDragState(HWND hwnd, BOOL fState)
{
  if(fState)
  {
    if(bDragState!=1) return false;
    bDragState++;
    SetCapture(hwnd);
  }
  else
  {
    if(bDragState!=2) return false;
    bDragState = 0;
    ReleaseCapture();
  }

  return true;
}

// -----------------------------------------------------------------------------------------------
// mouse move
BOOL OutputWnd::OnMouseMove(HWND hwnd)
{
  if(bDragState!=2) return false;

  POINT pnt;
  if( !GetCursorPos(&pnt) ) return false;

  RECT rect;
  GetWindowRect(hwnd, &rect);
  theApp.settings.nOutputWndHeight = rect.bottom - pnt.y;

  if(theApp.settings.nOutputWndHeight < OUTPUT_HEIGHT_MIN)
    theApp.settings.nOutputWndHeight = OUTPUT_HEIGHT_MIN;
  GetClientRect(theApp.hwndMain, &rect); pnt.y = rect.bottom;
  GetClientRect(theApp.hwndToolBar, &rect); pnt.y-=rect.bottom;
  pnt.y-=GetSystemMetrics(SM_CYMENUSIZE);
  if(theApp.settings.nOutputWndHeight > pnt.y-OUTPUT_HEIGHT_MIN)
    theApp.settings.nOutputWndHeight = pnt.y-OUTPUT_HEIGHT_MIN;

  theApp.onSize();
  UpdateWindow(theApp.hwndMain);
  return true;
}

// -----------------------------------------------------------------------------------------------
// cursor over window
BOOL OutputWnd::OnSetCursor(HWND hwnd)
{
  POINT pnt;
  if( !GetCursorPos(&pnt) ) return false;

  RECT rect;
  GetWindowRect(hwnd, &rect);
  if( pnt.y >= rect.top && pnt.y <= rect.top+OUTPUT_SPACE_T )
  {
    SetCursor( hCursor );
    bDragState = 1;
    return true;
  }

  bDragState = 0;
  return false;
}

// -----------------------------------------------------------------------------------------------
// resize window
BOOL OutputWnd::OnSize(HWND hwnd, LPARAM lParam)
{
  SetWindowPos(hwndTree, HWND_TOP,
    OUTPUT_SPACE_L, OUTPUT_SPACE_T,
    LOWORD(lParam)-OUTPUT_SPACE_L-OUTPUT_SPACE_R,
    HIWORD(lParam)-OUTPUT_SPACE_T-OUTPUT_SPACE_B,
    SWP_NOACTIVATE|SWP_NOZORDER
  );
  return false;
}


// -----------------------------------------------------------------------------------------------