// ==============================================================================================-
// Workspace.h : header file for Workspace class
// ==============================================================================================-

#if !defined(_WORKSPACE_H_)
#define _WORKSPACE_H_


// ==============================================================================================-
// definitions
//

#define WORKSPACE_SPACE_L 0
#define WORKSPACE_SPACE_R 5
#define WORKSPACE_SPACE_T 0
#define WORKSPACE_SPACE_B 0

#define WORKSPACE_WIDTH_MIN (5+5)

enum
{
  PRJITM_FILE = 0,
  PRJITM_NB_FILE,     //file (not build, eg. text-file)
  PRJITM_FOLDER = -1
};

typedef struct
{
  
  DWORD dwPrjItmID; //file (text,c++,asm,...), folder
  LPVOID *lpNext;   //points to next project-item (same layer), or NULL
  LPVOID *lpSub;    //points to next project-item (sub-layer), or NULL

  char szItmPath[PATH_SIZE];
  char szItmName[NAME_SIZE];
  char szItmArgs[ARGS_SIZE];

  SYSTEMTIME BuildTime;

  LPVOID lpSubData;     //additional storage (for later use)
  DWORD dwSizeSubData;

  //local
  HTREEITEM hTree;

} PROJECT_FILE;


typedef struct
{

  SYSTEMTIME BuildTime;

  LPVOID lpSubData;     //additional storage (for later use)
  DWORD dwSizeSubData;

} PROJECT_DATA;


typedef struct
{
  
  LPVOID *lpNext;   //points to next project, or NULL
  
  char szPrjPath[PATH_SIZE];
  char szPrjName[NAME_SIZE];

  PROJECT_DATA *PrjData;
  PROJECT_FILE *PrjFile;

  //local
  HTREEITEM hTree;

} WORKSPACE_DATA;


// ==============================================================================================-
// global functions
//

LRESULT CALLBACK WorkspaceWndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);


// ==============================================================================================-
// class Workspace : implemented in MainApp.cpp
//

class Workspace
{
// -----------------------------------------------------------------------------------------------
// methods

public:
  BOOL loadWorkspaceFile(LPSTR lpszFileName);
  BOOL saveWorkspaceFile();
  void freeWorkspace();

  Workspace(HWND hwnd);
  ~Workspace();

  BOOL SetDragState(HWND hwnd, BOOL fState);
  BOOL OnMouseMove(HWND hwnd);
  BOOL OnSetCursor(HWND hwnd);
  BOOL OnSize(HWND hwnd, LPARAM lParam);

// -----------------------------------------------------------------------------------------------
// data

public:
  BOOL fFailed;
  HWND hwndParent;
  HWND hwndTree;        //handle of list-view

  BYTE bDragState;
  HCURSOR hCursor;

  BOOL fLoaded;                     //a workspace is loaded
  LPSTR lpszWsFile;                 //path of workspace-file
  WORKSPACE_DATA *WorkspaceData;    //points to the first element of loaded workspace
};


#endif